import requests
import json
import time
import logging
import pkg_resources


from typing import Dict, Any

from requests import Response
from http import HTTPStatus

GUARDIAN_CLIENT_SDK_VERSION = pkg_resources.get_distribution("guardian-client").version

from guardian_client.python.credentials import GuardianClientCredentialContext


class GuardianAPIClient:
    """
    Client for Guardian API
    """

    def __init__(
        self,
        base_url: str,
        scan_endpoint: str = "scans",
        api_version: str = "v1",
        log_level: str = "INFO",
    ) -> None:
        """
        Initializes the Guardian API client.
        Args:
            base_url (str): The base URL of the Guardian API.
            scan_endpoint (str, optional): The endpoint for scanning. Defaults to "scans".
            api_version (str, optional): The API version. Defaults to "v1".
            log_level (str, optional): The log level. Defaults to "INFO".
        Raises:
            ValueError: If the log level is not one of "DEBUG", "INFO", "ERROR", or "CRITICAL".
        """
        self.endpoint = f"{base_url.rstrip('/')}/{api_version}/{scan_endpoint}"
        log_string_to_level = {
            "DEBUG": logging.DEBUG,
            "INFO": logging.INFO,
            "ERROR": logging.ERROR,
            "CRITICAL": logging.CRITICAL,
        }

        log_level_enum = log_string_to_level.get(log_level.upper(), logging.INFO)
        logging.basicConfig(
            level=log_level_enum,
            format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        )

        # client credential context is tied to a client instance.
        # In it's current state, a new client instance is created on each new call
        # to the guardian scanner, so a new context (and consequently a token)
        # is created for each scan request.
        self._access_token_context = GuardianClientCredentialContext()

    def scan(self, model_uri: str, poll_interval_secs: int = 5) -> Dict[str, Any]:
        """
        Submits a scan request for the given URI and polls for the scan status until it is completed.

        Args:
            uri (str): The URI to be scanned.
            poll_interval_secs (int, optional): The interval in seconds to poll for the scan status.
                If <= 0, the function returns immediately after submitting the scan. Defaults to 5.

        Returns:
            dict: A dictionary containing the HTTP status code and the scan status JSON.
                  If an error occurs during the scan submission or polling, the dictionary
                  will also contain the error details.
        """
        if not model_uri:
            logging.error("Model URI is required")
            return {
                "http_status_code": None,
                "error": "Model URI is required",
            }

        logging.info(f"Submitting scan for {model_uri}")

        headers = {
            "Authorization": f"Bearer {self._access_token_context.access_token}",
            "User-Agent": f"guardian-sdk/{GUARDIAN_CLIENT_SDK_VERSION}",
        }
        response = requests.post(
            self.endpoint,
            json={"model_uri": model_uri},
            headers=headers,
        )
        if response.status_code != HTTPStatus.ACCEPTED:
            return {
                "http_status_code": response.status_code,
                "error": self._decode_error(response),
            }

        logging.info(
            f"Scan submitted successfully for {model_uri} with status_code: {response.status_code}"
        )

        if poll_interval_secs <= 0:
            return {
                "http_status_code": response.status_code,
                "scan_status_json": response.json(),
            }

        response_json = response.json()
        id = response_json["uuid"]

        # Polling
        scan_status_json = None
        status_response = None

        logging.info(f"Polling for scan outcome for {id} for {model_uri}")
        while True:
            # reload header to check if token is still valid during this processing.
            headers = {
                "Authorization": f"Bearer {self._access_token_context.access_token}",
                "User-Agent": f"guardian-sdk/{GUARDIAN_CLIENT_SDK_VERSION}",
            }

            status_response = requests.get(
                url=f"{self.endpoint}/{id}",
                headers=headers,
            )
            if status_response.status_code == HTTPStatus.OK:
                scan_status_json = status_response.json()
                if scan_status_json["aggregate_eval_outcome"] != "PENDING":
                    break
            else:
                return {
                    "http_status_code": status_response.status_code,
                    "error": self._decode_error(status_response),
                }

            logging.debug(
                f"Scan outcome for {id} is {scan_status_json['aggregate_eval_outcome']}. Sleeping for 5 seconds before next check"
            )
            time.sleep(poll_interval_secs)  # Wait for 5 seconds before next check

        logging.info(f"Scan complete for {id} for {model_uri}")

        return {
            "http_status_code": (
                status_response.status_code if status_response else None
            ),
            "scan_status_json": scan_status_json,
        }

    def get_scan(self, scan_uuid: str) -> Dict[str, Any]:
        """
        Retrieves the scan results for a given past scan.

        Args:
            scan_uuid (str): The ID of the scan to retrieve.

        Returns:
            dict: A dictionary containing the HTTP status code and the scan status JSON.
                  If an error occurred during the scan, the dictionary
                  will contain the error details instead of the scan status.
        """
        if not scan_uuid:
            logging.error("Scan UUID is required")
            return {
                "http_status_code": None,
                "error": "Scan UUID is required",
            }

        logging.info(f"Retrieving scan ID {scan_uuid}")

        # reload header to check if token is still valid during this processing.
        headers = {
            "Authorization": f"Bearer {self._access_token_context.access_token}",
            "User-Agent": f"guardian-sdk/{GUARDIAN_CLIENT_SDK_VERSION}",
        }

        status_response = requests.get(
            url=f"{self.endpoint}/{scan_uuid}",
            headers=headers,
        )
        if status_response.status_code == HTTPStatus.OK:
            scan_status_json = status_response.json()
            return {
                "http_status_code": (
                    status_response.status_code if status_response else None
                ),
                "scan_status_json": scan_status_json,
            }
        else:
            return {
                "http_status_code": status_response.status_code,
                "error": self._decode_error(status_response),
            }

    def _decode_error(self, response: Response) -> str:
        try:
            response_json = response.json()
            if "detail" in response_json and response_json["detail"]:
                if isinstance(response_json["detail"], list):
                    concat_msg = ""
                    for item_ in response_json["detail"]:
                        concat_msg += f"- {item_['msg']}\n"
                    return concat_msg
                elif isinstance(response_json["detail"], str):
                    return response_json["detail"]

            return "Unknown error"
        except json.JSONDecodeError:
            return "Response is not in JSON format"



import json
import sys
import os

from guardian_client.python.api import GuardianAPIClient
import click
import pkg_resources

client_version = pkg_resources.get_distribution("guardian-client").version


GUARDIAN_ENDPOINT = os.getenv("GUARDIAN_ENDPOINT")
LOW, MEDIUM, HIGH, CRITICAL = "low", "medium", "high", "critical"
CRITICAL, ERROR, INFO, DEBUG = "critical", "error", "info", "debug"


@click.group()
@click.version_option(client_version, "-v", "--version")
@click.option(
    "--base-url",
    default=GUARDIAN_ENDPOINT,
    help="Base URL of the Guardian API. Overrides environment variable if set.",
)
@click.option(
    "--block-on-scan-errors",
    is_flag=True,
    help="Block if the scanning process failed on an incomplete scan",
)
@click.option(
    "--report-only",
    is_flag=True,
    help="Generate a JSON report only without evaluating scan results",
)
@click.option("--silent", is_flag=True, help="Do not print anything to stdout")
@click.option(
    "--log-level",
    default=INFO,
    type=str,
    required=False,
    help="Logging level if not silent (critical, error, info, debug)",
)
@click.pass_context
def cli(
    ctx,
    base_url: str,
    block_on_scan_errors: bool = True,
    report_only: bool = False,
    silent: bool = True,
    log_level: str = INFO,
) -> None:

    # Skip client initialization if help or version is requested
    if "--help" in sys.argv or "--version" in sys.argv or "-v" in sys.argv:
        return

    # In all cases other than help mode, base_url is required
    elif not base_url:
        click.echo("Error: Base URL is required", err=True)
        sys.exit(2)

    # Safety check that we are in CLI mode
    ctx.ensure_object(dict)

    guardian = GuardianAPIClient(
        base_url, log_level=log_level if not silent else CRITICAL
    )

    ctx.obj["SILENT"] = silent
    ctx.obj["BLOCK_ON_ERRORS"] = block_on_scan_errors
    ctx.obj["REPORT_ONLY"] = report_only
    ctx.obj["CLIENT"] = guardian


@cli.command()
@click.pass_context
@click.argument("model-uri", required=True)
@click.option(
    "--poll-interval-secs",
    default=5,
    type=int,
    required=False,
    help="Seconds to poll for scan status. If <= 0, the function returns immediately after submitting the scan",
)
def scan(
    ctx,
    model_uri: str,
    poll_interval_secs: int = 5,
) -> None:
    """
    Submits a scan request for the given URI
    """

    try:
        guardian: GuardianAPIClient = ctx.obj["CLIENT"]
        response = guardian.scan(model_uri, poll_interval_secs=poll_interval_secs)

        http_status_code = response.get("http_status_code")
        if not http_status_code or http_status_code // 100 != 2:
            click.echo(
                f"Error: Scan failed with status code: {http_status_code}, message: {response.get('error')}",
                err=True,
            )
            sys.exit(2)

        if not ctx.obj["SILENT"]:
            click.echo(json.dumps(response["scan_status_json"], indent=4))

        if ctx.obj["REPORT_ONLY"] or poll_interval_secs <= 0:
            sys.exit(0)

        if (
            response["scan_status_json"]["aggregate_eval_outcome"] == "ERROR"
            and ctx.obj["BLOCK_ON_ERRORS"]
        ):
            click.echo(
                f"Error: Scan failed with code: {response['scan_status_json']['error_code']}, message: {response['scan_status_json']['error_message']}",
                err=True,
            )
            sys.exit(2)

        if response["scan_status_json"]["aggregate_eval_outcome"] == "FAIL":
            click.echo(
                f"Error: Scan failed because it failed your organization's security policies",
                err=True,
            )
            sys.exit(1)
    except ValueError as e:
        click.echo(f"Error: Invalid arguments {e}", err=True)
        sys.exit(2)
    except Exception as e:
        click.echo(f"Error: Scan submission failed: {e}", err=True)
        sys.exit(2)

    sys.exit(0)


@cli.command()
@click.pass_context
@click.argument("scan-id", required=True)
def get_scan(ctx, scan_id: str):
    guardian: GuardianAPIClient = ctx.obj["CLIENT"]
    response = guardian.get_scan(scan_id)

    http_status_code = response.get("http_status_code")
    if (
        not http_status_code
        or http_status_code != 200
        or not response.get("scan_status_json")
    ):
        click.echo(
            f"Error: Scan retrieval failed with status code: {http_status_code}, message: {response.get('error')}",
            err=True,
        )
        sys.exit(2)

    if not ctx.obj["SILENT"]:
        click.echo(json.dumps(response["scan_status_json"], indent=4))

    if ctx.obj["REPORT_ONLY"]:
        sys.exit(0)

    if (
        response["scan_status_json"]["aggregate_eval_outcome"] == "ERROR"
        and ctx.obj["BLOCK_ON_ERRORS"]
    ):
        click.echo(
            f"Error: Scan failed with code: {response['scan_status_json']['error_code']}, message: {response['scan_status_json']['error_message']}",
            err=True,
        )
        sys.exit(2)

    if response["scan_status_json"]["aggregate_eval_outcome"] == "FAIL":
        click.echo(
            f"Error: Scan failed because it failed your organization's security policies",
            err=True,
        )
        sys.exit(1)
    sys.exit(0)


if __name__ == "__main__":
    cli(obj={})



import os
from datetime import datetime, timedelta, timezone

import jwt

import requests
import logging

from typing import Optional

logger = logging.getLogger(__name__)


class GuardianClientInvalidConfigException(Exception):
    """
    Raised when required environment variables for the client are missing.
    """


class InvalidAuthTokenException(Exception):
    """
    Raised when we fail to obtain an access token from the auth provider
    """


class GuardianClientCredentialContext:
    GUARDIAN_SCANNER_CLIENT_ID = "GUARDIAN_SCANNER_CLIENT_ID"
    GUARDIAN_SCANNER_CLIENT_SECRET = "GUARDIAN_SCANNER_CLIENT_SECRET"
    GUARDIAN_SCANNER_OIDP_TOKEN_ENDPOINT = "GUARDIAN_SCANNER_OIDP_TOKEN_ENDPOINT"

    def __init__(
        self,
    ) -> None:
        logger.info(f"New guardian-client context - fetching a new token from OIDP")
        self._access_token = self._load_access_token()

    @property
    def access_token(self) -> str:
        if self._is_credential_expired():
            # request for a fresh token and return when done.
            logger.info(
                f"Existing access token is expired ; fetching a new token from OIDP"
            )
            self._access_token = self._load_access_token()

        return self._access_token

    def _load_access_token(self) -> str:
        # authentication credentials
        self._client_id = os.getenv(self.GUARDIAN_SCANNER_CLIENT_ID)
        self._client_secret = os.getenv(self.GUARDIAN_SCANNER_CLIENT_SECRET)
        self._token_endpoint = os.getenv(self.GUARDIAN_SCANNER_OIDP_TOKEN_ENDPOINT)

        for val, name in [
            (self._client_id, self.GUARDIAN_SCANNER_CLIENT_ID),
            (self._client_secret, self.GUARDIAN_SCANNER_CLIENT_SECRET),
            (self._token_endpoint, self.GUARDIAN_SCANNER_OIDP_TOKEN_ENDPOINT),
        ]:
            if not val:
                logger.error(f"Failed to read {name} from the environment")
                raise GuardianClientInvalidConfigException(
                    f"Failed to read {name} from the environment"
                )

        # SDK is authenticated using the Client Credential flow
        access_token = self._request_access_token_from_oidp(
            oidp_token_url=str(self._token_endpoint),
            client_id=str(self._client_id),
            client_secret=str(self._client_secret),
        )

        if access_token is None:
            logger.critical(
                f"""
                Failed to obtain an access token. Check the logs for more information. 
                Make sure env variables are properly set.
                """
            )
            raise InvalidAuthTokenException("Failed to obtain an access token")

        logger.debug("Obtained a valid access token from auth provider")

        return access_token

    def _is_credential_expired(self) -> bool:
        if not self._access_token:
            raise InvalidAuthTokenException("Access token is not valid")

        token_exp = self._get_token_expiration_timestamp(self._access_token)

        expiration_datetime = datetime.fromtimestamp(token_exp, tz=timezone.utc)
        return datetime.now(timezone.utc) > expiration_datetime - timedelta(seconds=10)
    
    def _get_token_expiration_timestamp(self, token: str) -> str:
        try:
            decoded_token = jwt.decode(
                token, options={"verify_signature": False}
            )

            return decoded_token.get("exp")
        except jwt.DecodeError as e:
            logger.critical("Failed to decode JWT access token", exc_info=e)
            raise InvalidAuthTokenException("Access token is not valid")

    def _request_access_token_from_oidp(
        self,
        oidp_token_url: str,
        client_id: str,
        client_secret: str,
    ) -> Optional[str]:
        """
        Retrieves a JWT access token using client credentials flow.

        Arguments:
            oidp_token_url: The URL for your auth provider
            client_id: The client ID
            client_secret: The client secret
        """
        payload = {
            "grant_type": "client_credentials",
            "client_id": client_id,
            "client_secret": client_secret,
        }

        response = requests.post(
            oidp_token_url,
            data=payload,
            timeout=2,
        )

        access_token = response.json().get("access_token")
        if access_token and type(access_token) is str:
            return access_token
        else:
            raise InvalidAuthTokenException(
                "Access token requests failed; check if client credentials are correct"
            )
